import pytest
from pathlib import Path
import tempfile
import shutil
from gcspath import use_fs


@pytest.fixture()
def with_fs():
    tmp_dir = tempfile.mkdtemp()
    yield Path(tmp_dir)
    shutil.rmtree(tmp_dir)
    # Turn off FS adapter
    use_fs(False)



import pytest
from gcspath import GCSPath


def test_not_supported(monkeypatch):
    monkeypatch.setattr(GCSPath._flavour, "is_supported", False)
    with pytest.raises(NotImplementedError):
        GCSPath()


def test_cwd():
    with pytest.raises(NotImplementedError):
        GCSPath.cwd()


def test_home():
    with pytest.raises(NotImplementedError):
        GCSPath.home()


def test_chmod():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.chmod(0o666)


def test_lchmod():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.lchmod(0o666)


def test_group():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.group()


def test_is_mount():
    assert not GCSPath("/fake-bucket/fake-key").is_mount()


def test_is_symlink():
    assert not GCSPath("/fake-bucket/fake-key").is_symlink()


def test_is_socket():
    assert not GCSPath("/fake-bucket/fake-key").is_socket()


def test_is_fifo():
    assert not GCSPath("/fake-bucket/fake-key").is_fifo()


def test_is_block_device():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.is_block_device()


def test_is_char_device():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.is_char_device()


def test_lstat():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.lstat()


def test_resolve():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.resolve()


def test_symlink_to():
    path = GCSPath("/fake-bucket/fake-key")
    with pytest.raises(NotImplementedError):
        path.symlink_to("file_name")



from pathlib import Path
from gcspath import get_fs_client, use_fs, BucketClientFS


def test_use_fs(with_fs: Path):
    assert get_fs_client() is None
    # Use the default path
    use_fs()
    client = get_fs_client()
    assert isinstance(client, BucketClientFS)
    assert client.root.exists()

    # Use with False disables the client
    use_fs(False)
    client = get_fs_client()
    assert client is None

    # Can use a given path
    use_fs(str(with_fs))
    client = get_fs_client()
    assert isinstance(client, BucketClientFS)
    assert client.root == with_fs

    use_fs(False)



import os
import sys
import pytest
from pathlib import Path
from gcspath import PureGCSPath


def test_repr():
    assert repr(PureGCSPath("setup.py")) == "PureGCSPath('setup.py')"
    assert str(PureGCSPath("setup.py")) == "setup.py"
    assert bytes(PureGCSPath("setup.py")) == b"setup.py"
    assert PureGCSPath("/usr/bin").as_posix() == "/usr/bin"


@pytest.mark.skipif(sys.version_info < (3, 6), reason="requires python3.6 or higher")
def test_fspath():
    assert os.fspath(PureGCSPath("/usr/bin")) == "/usr/bin"


def test_join_strs():
    assert PureGCSPath("foo", "some/path", "bar") == PureGCSPath("foo/some/path/bar")


def test_join_paths():
    assert PureGCSPath(Path("foo"), Path("bar")) == PureGCSPath("foo/bar")


def test_empty():
    assert PureGCSPath() == PureGCSPath(".")


def test_absolute_paths():
    assert PureGCSPath("/etc", "/usr", "lib64") == PureGCSPath("/usr/lib64")


def test_slashes_single_double_dots():
    assert PureGCSPath("foo//bar") == PureGCSPath("foo/bar")
    assert PureGCSPath("foo/./bar") == PureGCSPath("foo/bar")
    assert PureGCSPath("foo/../bar") == PureGCSPath("bar")
    assert PureGCSPath("../bar") == PureGCSPath("../bar")
    assert PureGCSPath("foo", "../bar") == PureGCSPath("bar")


def test_operators():
    assert PureGCSPath("/etc") / "init.d" / "apache2" == PureGCSPath(
        "/etc/init.d/apache2"
    )
    assert "/usr" / PureGCSPath("bin") == PureGCSPath("/usr/bin")


def test_parts():
    assert PureGCSPath("foo//bar").parts == ("foo", "bar")
    assert PureGCSPath("foo/./bar").parts == ("foo", "bar")
    assert PureGCSPath("foo/../bar").parts == ("bar",)
    assert PureGCSPath("../bar").parts == ("..", "bar")
    assert PureGCSPath("foo", "../bar").parts == ("bar",)
    assert PureGCSPath("/foo/bar").parts == ("/", "foo", "bar")


def test_drive():
    assert PureGCSPath("foo//bar").drive == ""
    assert PureGCSPath("foo/./bar").drive == ""
    assert PureGCSPath("foo/../bar").drive == ""
    assert PureGCSPath("../bar").drive == ""
    assert PureGCSPath("foo", "../bar").drive == ""
    assert PureGCSPath("/foo/bar").drive == ""


def test_root():
    assert PureGCSPath("foo//bar").root == ""
    assert PureGCSPath("foo/./bar").root == ""
    assert PureGCSPath("foo/../bar").root == ""
    assert PureGCSPath("../bar").root == ""
    assert PureGCSPath("foo", "../bar").root == ""
    assert PureGCSPath("/foo/bar").root == "/"


def test_anchor():
    assert PureGCSPath("foo//bar").anchor == ""
    assert PureGCSPath("foo/./bar").anchor == ""
    assert PureGCSPath("foo/../bar").anchor == ""
    assert PureGCSPath("../bar").anchor == ""
    assert PureGCSPath("foo", "../bar").anchor == ""
    assert PureGCSPath("/foo/bar").anchor == "/"


def test_parents():
    assert tuple(PureGCSPath("foo//bar").parents) == (
        PureGCSPath("foo"),
        PureGCSPath("."),
    )
    assert tuple(PureGCSPath("foo/./bar").parents) == (
        PureGCSPath("foo"),
        PureGCSPath("."),
    )
    assert tuple(PureGCSPath("foo/../bar").parents) == (PureGCSPath("."),)
    assert tuple(PureGCSPath("../bar").parents) == (PureGCSPath(".."), PureGCSPath("."))
    assert tuple(PureGCSPath("foo", "../bar").parents) == (PureGCSPath("."),)
    assert tuple(PureGCSPath("/foo/bar").parents) == (
        PureGCSPath("/foo"),
        PureGCSPath("/"),
    )


def test_parent():
    assert PureGCSPath("foo//bar").parent == PureGCSPath("foo")
    assert PureGCSPath("foo/./bar").parent == PureGCSPath("foo")
    assert PureGCSPath("foo/../bar").parent == PureGCSPath(".")
    assert PureGCSPath("../bar").parent == PureGCSPath("..")
    assert PureGCSPath("foo", "../bar").parent == PureGCSPath(".")
    assert PureGCSPath("/foo/bar").parent == PureGCSPath("/foo")
    assert PureGCSPath(".").parent == PureGCSPath(".")
    assert PureGCSPath("/").parent == PureGCSPath("/")


def test_name():
    assert PureGCSPath("my/library/setup.py").name == "setup.py"


def test_suffix():
    assert PureGCSPath("my/library/setup.py").suffix == ".py"
    assert PureGCSPath("my/library.tar.gz").suffix == ".gz"
    assert PureGCSPath("my/library").suffix == ""


def test_suffixes():
    assert PureGCSPath("my/library.tar.gar").suffixes == [".tar", ".gar"]
    assert PureGCSPath("my/library.tar.gz").suffixes == [".tar", ".gz"]
    assert PureGCSPath("my/library").suffixes == []


def test_stem():
    assert PureGCSPath("my/library.tar.gar").stem == "library.tar"
    assert PureGCSPath("my/library.tar").stem == "library"
    assert PureGCSPath("my/library").stem == "library"


def test_uri():
    assert PureGCSPath("/etc/passwd").as_uri() == "gs://etc/passwd"
    assert PureGCSPath("/etc/init.d/apache2").as_uri() == "gs://etc/init.d/apache2"
    assert PureGCSPath("/bucket/key").as_uri() == "gs://bucket/key"


def test_absolute():
    assert PureGCSPath("/a/b").is_absolute()
    assert not PureGCSPath("a/b").is_absolute()


def test_reserved():
    assert not PureGCSPath("/a/b").is_reserved()
    assert not PureGCSPath("a/b").is_reserved()


def test_joinpath():
    assert PureGCSPath("/etc").joinpath("passwd") == PureGCSPath("/etc/passwd")
    assert PureGCSPath("/etc").joinpath(PureGCSPath("passwd")) == PureGCSPath(
        "/etc/passwd"
    )
    assert PureGCSPath("/etc").joinpath("init.d", "apache2") == PureGCSPath(
        "/etc/init.d/apache2"
    )


def test_match():
    assert PureGCSPath("a/b.py").match("*.py")
    assert PureGCSPath("/a/b/c.py").match("b/*.py")
    assert not PureGCSPath("/a/b/c.py").match("a/*.py")
    assert PureGCSPath("/a.py").match("/*.py")
    assert not PureGCSPath("a/b.py").match("/*.py")
    assert not PureGCSPath("a/b.py").match("*.Py")


def test_relative_to():
    gcs_path = PureGCSPath("/etc/passwd")
    assert gcs_path.relative_to("/") == PureGCSPath("etc/passwd")
    assert gcs_path.relative_to("/etc") == PureGCSPath("passwd")
    with pytest.raises(ValueError):
        gcs_path.relative_to("/usr")


def test_with_name():
    gcs_path = PureGCSPath("/Downloads/pathlib.tar.gz")
    assert gcs_path.with_name("setup.py") == PureGCSPath("/Downloads/setup.py")
    gcs_path = PureGCSPath("/")
    with pytest.raises(ValueError):
        gcs_path.with_name("setup.py")


def test_with_suffix():
    gcs_path = PureGCSPath("/Downloads/pathlib.tar.gz")
    assert gcs_path.with_suffix(".bz2") == PureGCSPath("/Downloads/pathlib.tar.bz2")
    gcs_path = PureGCSPath("README")
    assert gcs_path.with_suffix(".txt") == PureGCSPath("README.txt")
    gcs_path = PureGCSPath("README.txt")
    assert gcs_path.with_suffix("") == PureGCSPath("README")



from dataclasses import dataclass, field
from typing import Optional, List, Generator
from .client import BucketClient, ClientBucket, ClientBlob, ClientError, BucketEntry
from .base import PureGCSPath

try:
    from google.cloud import storage
    from google.api_core import exceptions as gcs_errors

    has_gcs = True
except ImportError:
    storage = None
    has_gcs = False


class BucketEntryGCS(BucketEntry["ClientBucketGCS", storage.Blob]):
    ...


@dataclass
class ClientBlobGCS(ClientBlob["ClientBucketGCS", storage.Blob]):
    def delete(self) -> None:
        self.raw.delete()

    def exists(self) -> bool:
        return self.raw.exists()


@dataclass
class ClientBucketGCS(ClientBucket):
    name: str
    bucket: storage.Bucket

    def get_blob(self, blob_name: str) -> Optional[ClientBlobGCS]:
        native_blob = self.bucket.get_blob(blob_name)
        if native_blob is None:
            return None
        return ClientBlobGCS(
            bucket=self.bucket,
            owner=native_blob.owner,
            name=native_blob.name,
            raw=native_blob,
            size=native_blob.size,
            updated=native_blob.updated.timestamp(),
        )

    def copy_blob(
        self, blob: ClientBlobGCS, target: "ClientBucketGCS", name: str
    ) -> Optional[ClientBlobGCS]:
        assert blob.raw is not None, "raw storage.Blob instance required"
        native_blob = self.bucket.copy_blob(blob.raw, target.bucket, name)
        if native_blob is None:
            return None
        return ClientBlobGCS(
            bucket=self.bucket,
            owner=native_blob.owner,
            name=native_blob.name,
            raw=native_blob,
            size=native_blob.size,
            updated=native_blob.updated.timestamp(),
        )

    def delete_blob(self, blob: ClientBlobGCS) -> None:
        return self.bucket.delete_blob(blob.name)

    def delete_blobs(self, blobs: List[ClientBlobGCS]) -> None:
        return self.bucket.delete_blobs(blobs)


@dataclass
class BucketClientGCS(BucketClient):
    client: storage.Client = field(default_factory=lambda: storage.Client())

    def create_bucket(self, path: PureGCSPath) -> ClientBucket:
        return self.client.create_bucket(path.bucket_name)

    def delete_bucket(self, path: PureGCSPath) -> None:
        bucket = self.client.get_bucket(path.bucket_name)
        bucket.delete()

    def lookup_bucket(self, path: PureGCSPath) -> Optional[ClientBucketGCS]:
        try:
            native_bucket = self.client.lookup_bucket(path.bucket_name)
            if native_bucket is not None:
                return ClientBucketGCS(str(path.bucket_name), bucket=native_bucket)
        except gcs_errors.ClientError:
            pass
        return None

    def get_bucket(self, path: PureGCSPath) -> ClientBucketGCS:
        try:
            native_bucket = self.client.lookup_bucket(path.bucket_name)
            if native_bucket is not None:
                return ClientBucketGCS(str(path.bucket_name), bucket=native_bucket)
            raise FileNotFoundError(f"Bucket {path.bucket_name} does not exist!")
        except gcs_errors.ClientError as e:
            raise ClientError(message=e.message, code=e.code)

    def list_buckets(self, **kwargs) -> Generator[ClientBucket, None, None]:
        return self.client.list_buckets(**kwargs)

    def scandir(
        self,
        path: Optional[PureGCSPath] = None,
        prefix: Optional[str] = None,
        delimiter: Optional[str] = None,
        include_raw: bool = False,
    ) -> Generator[BucketEntryGCS, None, None]:
        continuation_token = None
        if path is None or not path.bucket_name:
            for bucket in self.list_buckets():
                yield BucketEntryGCS(bucket.name, is_dir=True, raw=None)
            return
        sep = path._flavour.sep
        bucket = self.lookup_bucket(path)
        if bucket is None:
            return
        while True:
            if continuation_token:
                response = self.client.list_blobs(
                    bucket.name,
                    prefix=prefix,
                    delimiter=sep,
                    page_token=continuation_token,
                )
            else:
                response = self.client.list_blobs(
                    bucket.name, prefix=prefix, delimiter=sep
                )
            for page in response.pages:
                for folder in list(page.prefixes):
                    full_name = folder[:-1] if folder.endswith(sep) else folder
                    name = full_name.split(sep)[-1]
                    yield BucketEntryGCS(name, is_dir=True, raw=None)
                for item in page:
                    yield BucketEntryGCS(
                        name=item.name.split(sep)[-1],
                        is_dir=False,
                        size=item.size,
                        last_modified=item.updated.timestamp(),
                        raw=item,
                    )
            if response.next_page_token is None:
                break
            continuation_token = response.next_page_token

    def list_blobs(
        self,
        path: PureGCSPath,
        prefix: Optional[str] = None,
        delimiter: Optional[str] = None,
        include_dirs: bool = False,
    ) -> Generator[ClientBlobGCS, None, None]:
        continuation_token = None
        bucket = self.lookup_bucket(path)
        if bucket is None:
            return
        while True:
            if continuation_token:
                response = self.client.list_blobs(
                    path.bucket_name,
                    prefix=prefix,
                    delimiter=delimiter,
                    page_token=continuation_token,
                )
            else:
                response = self.client.list_blobs(
                    path.bucket_name, prefix=prefix, delimiter=delimiter
                )
            for page in response.pages:
                for item in page:
                    yield ClientBlobGCS(
                        bucket=bucket,
                        owner=item.owner,
                        name=item.name,
                        raw=item,
                        size=item.size,
                        updated=item.updated.timestamp(),
                    )
            if response.next_page_token is None:
                break
            continuation_token = response.next_page_token



from .api import GCSPath, PureGCSPath  # noqa
from .client import BucketEntry, BucketStat  # noqa
from .api import (  # noqa
    get_fs_client,
    use_fs,
    BucketClient,
    ClientBlob,
    ClientBucket,
    ClientError,
)
from .file import BucketClientFS, BucketEntryFS, ClientBlobFS, ClientBucketFS  # noqa
from .gcs import BucketClientGCS, BucketEntryGCS, ClientBlobGCS, ClientBucketGCS  # noqa



from dataclasses import dataclass, field
from typing import Optional, List, Generator, cast
from .client import BucketClient, ClientBucket, ClientBlob, ClientError, BucketEntry
from .base import PureGCSPath
import pathlib
import shutil
import os


class BucketEntryFS(BucketEntry["ClientBucketFS", pathlib.Path]):
    ...


@dataclass
class ClientBlobFS(ClientBlob["ClientBucketFS", pathlib.Path]):
    raw: pathlib.Path
    bucket: "ClientBucketFS"

    def delete(self) -> None:
        """Delete a file-based blob."""
        file_folder: str = os.path.dirname(self.raw)
        self.raw.unlink()
        # NOTE: in buckets folders only exist if a file is contained in them. Mimic
        # that behavior here by removing empty folders when the last file is removed.
        if len(os.listdir(file_folder)) == 0:
            shutil.rmtree(file_folder)

    def exists(self) -> bool:
        return self.raw.exists()


@dataclass
class ClientBucketFS(ClientBucket):
    name: str
    bucket: pathlib.Path

    def get_blob(self, blob_name: str) -> Optional[ClientBlobFS]:
        native_blob = self.bucket / blob_name
        if not native_blob.exists() or native_blob.is_dir():
            return None
        stat = native_blob.stat()
        return ClientBlobFS(
            bucket=self,
            owner=native_blob.owner(),
            name=blob_name,
            raw=native_blob,
            size=stat.st_size,
            updated=int(round(stat.st_mtime_ns * 1000)),
        )

    def copy_blob(
        self, blob: ClientBlobFS, target: "ClientBucketFS", name: str
    ) -> Optional[ClientBlobFS]:
        in_file = str(blob.bucket.bucket / blob.name)
        out_file = str(target.bucket / name)
        out_path = pathlib.Path(os.path.dirname(out_file))
        if not out_path.exists():
            out_path.mkdir(parents=True)
        shutil.copy(in_file, out_file)
        return None

    def delete_blob(self, blob: ClientBlobFS) -> None:
        blob.delete()

    def delete_blobs(self, blobs: List[ClientBlobFS]) -> None:
        for blob in blobs:
            blob.delete()


@dataclass
class BucketClientFS(BucketClient):
    # Root to store file-system buckets as children of
    root: pathlib.Path

    def full_path(self, path: PureGCSPath) -> pathlib.Path:
        if path.bucket_name is None:
            raise ValueError(f"Invalid bucket name for path: {path}")
        full_path = self.root.absolute() / path.bucket_name
        if path.key is not None:
            full_path = full_path / path.key
        return full_path

    def exists(self, path: PureGCSPath) -> bool:
        """Return True if the path exists as a file or folder on disk"""
        return self.full_path(path).exists()

    def is_dir(self, path: PureGCSPath) -> bool:
        return self.full_path(path).is_dir()

    def rmdir(self, path: PureGCSPath) -> None:
        full_path = self.full_path(path)
        return shutil.rmtree(str(full_path))

    def open(
        self,
        path: PureGCSPath,
        *,
        mode="r",
        buffering=-1,
        encoding=None,
        errors=None,
        newline=None,
    ):
        if self.lookup_bucket(path) is None:
            raise ClientError(
                message=f'bucket "{path.bucket_name}" does not exist', code=404
            )

        full_path = self.full_path(path)
        if not full_path.exists():
            if full_path.suffix != "":
                str_full = str(full_path)
                full_path = pathlib.Path(str_full[: str_full.rindex("/") + 1])
            full_path.mkdir(parents=True, exist_ok=True)
        return super().open(
            path,
            mode=mode,
            buffering=buffering,
            encoding=encoding,
            errors=errors,
            newline=newline,
        )

    def make_uri(self, path: PureGCSPath) -> str:
        if not path.bucket_name:
            raise ValueError(
                f"cannot make a URI to an invalid bucket: {path.bucket_name}"
            )
        result = f"file://{self.root.absolute() / path.bucket_name / path.key}"
        return result

    def create_bucket(self, path: PureGCSPath) -> ClientBucket:
        if not path.bucket_name:
            raise ValueError(f"Invalid bucket name: {path.bucket_name}")
        bucket_path: pathlib.Path = self.root / path.bucket_name
        if bucket_path.exists():
            raise FileExistsError(f"Bucket already exists at: {bucket_path}")
        bucket_path.mkdir(parents=True, exist_ok=True)
        return ClientBucketFS(str(path.bucket_name), bucket=bucket_path)

    def delete_bucket(self, path: PureGCSPath) -> None:
        bucket_path: pathlib.Path = self.root / str(path.bucket_name)
        if bucket_path.exists():
            shutil.rmtree(bucket_path)

    def lookup_bucket(self, path: PureGCSPath) -> Optional[ClientBucketFS]:
        if path.bucket_name:
            bucket_path: pathlib.Path = self.root / path.bucket_name
            if bucket_path.exists():
                return ClientBucketFS(str(path.bucket_name), bucket=bucket_path)
        return None

    def get_bucket(self, path: PureGCSPath) -> ClientBucketFS:
        if not path.bucket_name:
            raise ValueError(f"path has an invalid bucket_name: {path.bucket_name}")
        bucket_path: pathlib.Path = self.root / path.bucket_name
        if bucket_path.is_dir():
            return ClientBucketFS(str(path.bucket_name), bucket=bucket_path)
        raise FileNotFoundError(f"Bucket {path.bucket_name} does not exist!")

    def list_buckets(self, **kwargs) -> Generator[ClientBucketFS, None, None]:
        for f in self.root.glob("*"):
            if f.is_dir():
                yield ClientBucketFS(f.name, f)

    def scandir(
        self,
        path: Optional[PureGCSPath] = None,
        prefix: Optional[str] = None,
        delimiter: Optional[str] = None,
    ) -> Generator[BucketEntryFS, None, None]:
        if path is None or not path.bucket_name:
            for bucket in self.list_buckets():
                yield BucketEntryFS(bucket.name, is_dir=True, raw=None)
            return
        assert path is not None
        assert path.bucket_name is not None
        scan_path = self.root / path.bucket_name
        if prefix is not None:
            scan_path = scan_path / prefix
        for dir_entry in scan_path.glob("*"):
            if dir_entry.is_dir():
                yield BucketEntryFS(dir_entry.name, is_dir=True, raw=None)
            else:
                file_path = pathlib.Path(dir_entry)
                stat = file_path.stat()
                file_size = stat.st_size
                updated = int(round(stat.st_mtime_ns * 1000))
                blob: ClientBlob = ClientBlobFS(
                    self.get_bucket(path),
                    name=dir_entry.name,
                    size=file_size,
                    updated=updated,
                    owner=None,
                    raw=file_path,
                )
                yield BucketEntryFS(
                    name=dir_entry.name,
                    is_dir=False,
                    size=file_size,
                    last_modified=updated,
                    raw=blob,
                )

    def list_blobs(
        self,
        path: PureGCSPath,
        prefix: Optional[str] = None,
        delimiter: Optional[str] = None,
        include_dirs: bool = False,
    ) -> Generator[ClientBlobFS, None, None]:
        assert path.bucket_name is not None
        bucket = self.get_bucket(path)
        scan_path = self.root / path.bucket_name
        if prefix is not None:
            scan_path = scan_path / prefix
        elif prefix is not None:
            scan_path = scan_path / path.key

        # Path to a file
        if scan_path.exists() and not scan_path.is_dir():
            stat = scan_path.stat()
            file_size = stat.st_size
            updated = int(round(stat.st_mtime_ns * 1000))
            yield ClientBlobFS(
                bucket,
                name=scan_path.name,
                size=file_size,
                updated=updated,
                owner=None,
                raw=scan_path,
            )

        # If the path can't be scanned, yield nothing and return
        dir_entries_generator = scan_path.rglob("*")

        # Yield blobs for each file (non-folder)
        for file_path in dir_entries_generator:
            if file_path.is_dir():
                continue
            stat = file_path.stat()
            file_size = stat.st_size
            updated = int(round(stat.st_mtime_ns * 1000))
            yield ClientBlobFS(
                bucket,
                name=f"{prefix if prefix is not None else ''}{file_path.name}",
                size=file_size,
                updated=updated,
                owner=None,
                raw=file_path,
            )



from pathlib import PurePosixPath, PureWindowsPath

import pytest
from gcspath import PureGCSPath


def test_paths_of_a_different_flavour():
    with pytest.raises(TypeError):
        PureGCSPath("/bucket/key") < PurePosixPath("/bucket/key")

    with pytest.raises(TypeError):
        PureWindowsPath("/bucket/key") > PureGCSPath("/bucket/key")
